@@ -5,8 +5,8 @@
 
 ;; Author: Artem Khramov <akhramov+emacs@pm.me>
 ;; Created: 6 Jan 2017
-;; Package-Version: 20260123.231
-;; Package-Revision: ea0dddba6046
+;; Package-Version: 20260127.533
+;; Package-Revision: 9211d14128a1
 ;; Package-Requires: ((alert "1.2") (async "1.9.3") (dash "2.18.0") (emacs "27.1"))
 ;; Keywords: calendar, convenience
 ;; URL: https://github.com/emacsorphanage/org-wild-notifier.el
@@ -53,7 +53,7 @@
 (require 'cl-lib)
 
 (defgroup org-wild-notifier nil
-  "org-wild-notifier customization options"
+  "Customization options for org-wild-notifier."
   :group 'org)
 
 (defcustom org-wild-notifier-alert-time '(10)
@@ -144,16 +144,15 @@ Value should be space-separated timestamp types, e.g., \"DEADLINE SCHEDULED\"."
 (defcustom org-wild-notifier-predicate-whitelist nil
   "Receive notifications for events matching these predicates only.
 Each function should take an event POM and return non-nil iff that event should
-trigger a notification. Leave this variable blank if you do not want to filter
+trigger a notification.  Leave this variable blank if you do not want to filter
 anything."
   :package-version '(org-wild-notifier . "0.5.0")
   :group 'org-wild-notifier
   :type '(function))
 
 (defcustom org-wild-notifier-additional-environment-regexes nil
-  "Additional regular expressions that should be provided to
-`async-inject-environment' before running the async command to
-check notifications."
+  "Additional regular expressions for `async-inject-environment'.
+These are passed to the async command when checking notifications."
   :package-version '(org-wild-notifier . "0.5.0")
   :group 'org-wild-notifier
   :type '(string))
@@ -182,16 +181,20 @@ Options: high, medium, low."
   :type 'plist)
 
 (defcustom org-wild-notifier-day-wide-alert-times nil
-  "List of time strings when alerts for day-wide events should trigger."
+  "List of time strings when alerts for day-wide events should trigger.
+Each string should be in HH:MM format (e.g., \"09:00\", \"14:30\").
+Day-wide events (those without a specific time) will trigger notifications
+at each of these times."
   :package-version '(org-wild-notifier . "0.5.0")
   :group 'org-wild-notifier
-  :type 'string)
+  :type '(repeat string))
 
 (defcustom org-wild-notifier-show-any-overdue-with-day-wide-alerts t
-  "Show any overdue TODO items along with day wide alerts whenever they are shown."
+  "Show any overdue TODO items along with day-wide alerts.
+When non-nil, overdue items are included in day-wide alert notifications."
   :package-version '(org-wild-notifier . "0.5.0")
   :group 'org-wild-notifier
-  :type 'string)
+  :type 'boolean)
 
 (defvar org-wild-notifier--timer nil
   "Timer value.")
@@ -226,24 +229,31 @@ in order to ignore seconds."
    (time-add (current-time) (seconds-to-time (* 60 interval)))
    timestamp))
 
+(defun org-wild-notifier--notification-due-now-p (notification)
+  "Return non-nil if NOTIFICATION should fire at the current minute."
+  (org-wild-notifier--time= (plist-get notification :notify-at) (current-time)))
+
+(defun org-wild-notifier--get-current-notifications (event)
+  "Get notifications for EVENT that are due at the current minute.
+Returns a list of notification plists filtered to those due now."
+  (->> (org-wild-notifier--get-notifications-for-event event)
+       (--filter (org-wild-notifier--notification-due-now-p it))))
+
 (defun org-wild-notifier--notifications (event)
-  "Get notifications for given EVENT.
-Returns a list of time information interval pairs."
-  (->> (list
-        (org-wild-notifier--filter-day-wide-events (cadr (assoc 'times event)))
-        (cdr (assoc 'intervals event)))
-         (apply '-table-flat (lambda (ts int) (list ts int)))
-         ;; When no values are provided for table flat, we get the second values
-         ;; paired with nil.
-         (--filter (not (null (car it))))
-         (--filter (org-wild-notifier--timestamp-within-interval-p (cdar it) (cadr it)))))
+  "Get notifications for given EVENT that are due now.
+Returns a list of notification plists.
+DEPRECATED: Use `org-wild-notifier--get-current-notifications' instead."
+  (org-wild-notifier--get-current-notifications event))
 
 (defun org-wild-notifier--has-timestamp (s)
+  "Check whether S has a time component in its timestamp."
   (string-match org-ts-regexp0 s)
   (match-beginning 7))
 
 (defun org-wild-notifier--filter-day-wide-events (times)
-  (--filter (org-wild-notifier--has-timestamp (car it)) times))
+  "Filter out day-wide events (those without a time component) from TIMES.
+TIMES is a list of time plists with :timestamp-string."
+  (--filter (org-wild-notifier--has-timestamp (plist-get it :timestamp-string)) times))
 
 (defun org-wild-notifier--time-left (seconds)
   "Human-friendly representation for SECONDS."
@@ -269,18 +279,22 @@ Returns a list of time information interval pairs."
           (org-wild-notifier--time-left (* 60 (cdr str-interval)))))
 
 (defun org-wild-notifier-get-minutes-into-day (time)
+  "Return the number of minutes from midnight for TIME string."
   (org-duration-to-minutes (org-get-time-of-day time t)))
 
 (defun org-wild-notifier-get-hours-minutes-from-time (time-string)
+  "Parse TIME-STRING and return a list of (hours minutes)."
   (let ((total-minutes (truncate (org-wild-notifier-get-minutes-into-day time-string))))
     (list (/ total-minutes 60)
           (mod total-minutes 60))))
 
 (defun org-wild-notifier-set-hours-minutes-for-time (time hours minutes)
+  "Return TIME with HOURS and MINUTES set, preserving the date."
   (cl-destructuring-bind (_s _m _h day month year dow dst utcoff) (decode-time time)
     (encode-time 0 minutes hours day month year dow dst utcoff)))
 
 (defun org-wild-notifier-current-time-matches-time-of-day-string (time-of-day-string)
+  "Return non-nil if current time matches TIME-OF-DAY-STRING."
   (let ((now (current-time)))
     (org-wild-notifier--time=
      now
@@ -289,55 +303,75 @@ Returns a list of time information interval pairs."
             (org-wild-notifier-get-hours-minutes-from-time time-of-day-string)))))
 
 (defun org-wild-notifier-current-time-is-day-wide-time ()
+  "Return non-nil if current time matches any day-wide alert time."
   (--any (org-wild-notifier-current-time-matches-time-of-day-string it)
          org-wild-notifier-day-wide-alert-times))
 
+(defun org-wild-notifier--day-wide-alert-times-for-today ()
+  "Get Emacs time values for all day-wide alert times for today."
+  (let ((now (current-time)))
+    (--map (apply 'org-wild-notifier-set-hours-minutes-for-time
+                  now
+                  (org-wild-notifier-get-hours-minutes-from-time it))
+           org-wild-notifier-day-wide-alert-times)))
+
 (defun org-wild-notifier-day-wide-notifications (events)
+  "Return unique day-wide notification texts for EVENTS."
   (->> events
        (-filter 'org-wild-notifier-display-as-day-wide-event)
        (-map 'org-wild-notifier--day-wide-notification-text)
        (-uniq)))
 
 (defun org-wild-notifier-display-as-day-wide-event (event)
-  ;; `org-wild-notifier-event-has-any-passed-time' event is a requirement,
-  ;; regardless of whether
-  ;; `org-wild-notifier-show-any-overdue-with-day-wide-alerts' is set because
-  ;; the events list can include events scheduled tomorrow. We only want to
-  ;; alert for things scheduled today.
+  "Return non-nil if EVENT should display as a day-wide event.
+`org-wild-notifier-event-has-any-passed-time' is required regardless of
+`org-wild-notifier-show-any-overdue-with-day-wide-alerts' because the events
+list can include events scheduled tomorrow.  We only alert for today."
   (and (org-wild-notifier-event-has-any-passed-time event)
       (or org-wild-notifier-show-any-overdue-with-day-wide-alerts
            (org-wild-notifier-event-has-any-day-wide-timestamp event))))
 
 (defun org-wild-notifier-event-has-any-day-wide-timestamp (event)
-  (--any (not (org-wild-notifier--has-timestamp (car it)))
-         (car (cdr (assoc 'times event)))))
+  "Return non-nil if EVENT has any day-wide timestamp (no time component)."
+  (--any (not (org-wild-notifier--has-timestamp (plist-get it :timestamp-string)))
+         (cadr (assoc 'times event))))
 
 (defun org-wild-notifier-event-has-any-passed-time (event)
-  (--any (time-less-p (cdr it) (current-time))
-         (car (cdr (assoc 'times event )))))
+  "Return non-nil if EVENT has any timestamp that has already passed."
+  (--any (time-less-p (plist-get it :time) (current-time))
+         (cadr (assoc 'times event))))
 
 (defun org-wild-notifier--day-wide-notification-text (event)
   "For given STR-INTERVAL list and EVENT get notification wording."
   (format "%s is due or scheduled today"
           (cdr (assoc 'title event))))
 
+(defun org-wild-notifier--format-notification (notification)
+  "Format NOTIFICATION plist as a human-readable message string."
+  (let ((type (plist-get notification :type))
+        (title (plist-get notification :title))
+        (event-time-string (plist-get notification :event-time-string))
+        (minutes-before (plist-get notification :minutes-before)))
+    (pcase type
+      ('absolute (format "%s (scheduled reminder)" title))
+      ('day-wide (format "%s is due or scheduled today" title))
+      (_ (format "%s at %s (%s)"
+                 title
+                 (org-wild-notifier--get-hh-mm-from-org-time-string event-time-string)
+                 (org-wild-notifier--time-left (* 60 minutes-before)))))))
+
 (defun org-wild-notifier--check-notify-at (event)
   "Check if any absolute notification times in EVENT match now.
-Returns list of notification messages."
-  (--keep
-   (when (org-wild-notifier--time= it (current-time))
-     (format "%s (scheduled reminder)" (cdr (assoc 'title event))))
-   (cdr (assoc 'notify-at event))))
+Returns list of notification plists.
+DEPRECATED: Use `org-wild-notifier--get-current-notifications' instead."
+  (--filter (eq (plist-get it :type) 'absolute)
+            (org-wild-notifier--get-current-notifications event)))
 
 (defun org-wild-notifier--check-event (event)
-  "Get notifications for given EVENT.
-Returns a list of notification messages"
-  (append
-   ;; Relative time notifications
-   (->> (org-wild-notifier--notifications event)
-        (--map (org-wild-notifier--notification-text `(,(caar it) . ,(cadr it)) event)))
-   ;; Absolute time notifications
-   (org-wild-notifier--check-notify-at event)))
+  "Get notifications for given EVENT that are due now.
+Returns a list of notification messages."
+  (->> (org-wild-notifier--get-current-notifications event)
+       (--map (org-wild-notifier--format-notification it))))
 
 (defun org-wild-notifier--get-tags (marker)
   "Retrieve tags of MARKER."
@@ -346,6 +380,7 @@ Returns a list of notification messages"
       (org-split-string  ":")))
 
 (defun org-wild-notifier--whitelist-predicates ()
+  "Return a list of predicate functions for whitelist filtering."
   (->> `([,org-wild-notifier-keyword-whitelist
           (lambda (it)
             (-contains-p org-wild-notifier-keyword-whitelist
@@ -363,6 +398,7 @@ Returns a list of notification messages"
        (--map (aref it 1))))
 
 (defun org-wild-notifier--blacklist-predicates ()
+  "Return a list of predicate functions for blacklist filtering."
   (->> `([,org-wild-notifier-keyword-blacklist
           (lambda (it)
             (-contains-p org-wild-notifier-keyword-blacklist
@@ -422,6 +458,7 @@ Returns a list of notification messages"
 
 
 (defun org-wild-notifier-environment-regex ()
+  "Return a regex matching all environment variables to inject."
   (macroexpand
    `(rx (or
          ,@(mapcar (lambda (regexp) (list 'regexp regexp))
@@ -486,14 +523,17 @@ First checks for a per-entry override via `org-wild-notifier-entries-property'
 
 (defun org-wild-notifier--extract-time (marker)
   "Extract timestamps from MARKER.
-Timestamps are extracted as cons cells.  car holds org-formatted
-string, cdr holds time in list-of-integer format."
+Returns a list of plists, each containing:
+  :timestamp-type - symbol: deadline, scheduled, or timestamp
+  :timestamp-string - the original org timestamp string
+  :time - parsed time in Emacs time format"
   (-non-nil
    (--map
     (let ((org-timestamp (org-entry-get marker it)))
       (and org-timestamp
-           (cons org-timestamp
-                 (org-wild-notifier--timestamp-parse org-timestamp))))
+           (list :timestamp-type (intern (downcase it))
+                 :timestamp-string org-timestamp
+                 :time (org-wild-notifier--timestamp-parse org-timestamp))))
     (org-wild-notifier--get-entries-for-marker marker))))
 
 (defun org-wild-notifier--extract-title (marker)
@@ -526,9 +566,9 @@ rather than the real system time."
                                  (decoded-time-day parsed)
                                  (decoded-time-month parsed)
                                  (decoded-time-year parsed))))
-    ;; Handle repeating timestamps by finding the closest occurrence to current-time
+    ;; Handle repeating timestamps by finding the closest occurrence to now
     (if (string-match "\\+[0-9]+[dwmy]" timestamp)
-        ;; Repeating timestamp - find closest occurrence to (current-time)
+        ;; Repeating timestamp - find closest occurrence to now
         (let* ((repeat-match (match-string 0 timestamp))
                (repeat-num (string-to-number (substring repeat-match 1 -1)))
                (repeat-unit (substring repeat-match -1))
@@ -565,16 +605,33 @@ in the WILD_NOTIFIER_NOTIFY_AT property."
           (setq start match-end-pos)))
       (nreverse timestamps))))
 
+(defun org-wild-notifier--extract-marker-location (marker)
+  "Extract file, position, and ID from MARKER.
+Returns an alist with file, pos, and optionally id."
+  (when (and marker (marker-buffer marker))
+    (with-current-buffer (marker-buffer marker)
+      (save-excursion
+        (goto-char (marker-position marker))
+        (let ((file (buffer-file-name))
+              (pos (marker-position marker))
+              (id (org-entry-get nil "ID")))
+          `((file . ,file)
+            (pos . ,pos)
+            ,@(when id `((id . ,id)))))))))
+
 (defun org-wild-notifier--gather-info (marker)
   "Collect information about an event.
 MARKER acts like event's identifier."
-  `((times . (,(org-wild-notifier--extract-time marker)))
-    (title . ,(org-wild-notifier--extract-title marker))
-    (intervals . ,(org-wild-notifier--extract-notication-intervals marker))
-    (notify-at . ,(org-wild-notifier--extract-notify-at-times marker))))
+  (let ((location (org-wild-notifier--extract-marker-location marker)))
+    `((marker . ,marker)
+      (times . (,(org-wild-notifier--extract-time marker)))
+      (title . ,(org-wild-notifier--extract-title marker))
+      (intervals . ,(org-wild-notifier--extract-notication-intervals marker))
+      (notify-at . ,(org-wild-notifier--extract-notify-at-times marker))
+      ,@location)))
 
 (defun org-wild-notifier--stop ()
-  "Stops the notification timer and cancel any in-progress checks."
+  "Stop the notification timer and cancel any in-progress check."
   (-some-> org-wild-notifier--timer (cancel-timer))
   (when org-wild-notifier--process
     (interrupt-process org-wild-notifier--process)
@@ -592,6 +649,7 @@ smoother experience this function also runs a check without timer."
        (setf org-wild-notifier--timer it)))
 
 (defun org-wild-notifier--check-events (events)
+  "Process EVENTS and send notifications for those due now."
   (setq org-wild-notifier--process nil)
   (-each
       (->> events
@@ -599,9 +657,6 @@ smoother experience this function also runs a check without timer."
            (-flatten)
            (-uniq))
     'org-wild-notifier--notify)
-  (when (org-wild-notifier-current-time-is-day-wide-time)
-    (-each (org-wild-notifier-day-wide-notifications events)
-      #'org-wild-notifier--notify))
   (setq org-wild-notifier--last-check-time (current-time)))
 
 ;;;###autoload
@@ -669,6 +724,167 @@ TIME is a string in HH:MM format or any format accepted by `org-read-date'."
     (message "Notification scheduled for \"%s\" at %s"
              title (format-time-string "%Y-%m-%d %H:%M" target-time))))
 
+(defun org-wild-notifier--get-notifications-for-event (event)
+  "Get all notifications for EVENT.
+Returns a list of notification plists, each containing:
+  :notify-at - Emacs time when notification should fire
+  :event-time - Emacs time of the event (nil for absolute/day-wide)
+  :event-time-string - Original timestamp string (nil for absolute/day-wide)
+  :timestamp-type - deadline, scheduled, or timestamp (nil for abs/day-wide)
+  :minutes-before - Minutes before event (nil for absolute/day-wide)
+  :type - Symbol: relative, absolute, or day-wide
+  :title - Event title string
+  :marker - Org marker for navigation/queries
+  :file - Source org file path
+  :pos - Position in file
+  :id - Org-mode ID property (if set)
+  :event - The full event alist"
+  (let ((notifications nil)
+        (marker (cdr (assoc 'marker event)))
+        (title (cdr (assoc 'title event)))
+        (times (cadr (assoc 'times event)))
+        (intervals (cdr (assoc 'intervals event)))
+        (notify-at-times (cdr (assoc 'notify-at event)))
+        (file (cdr (assoc 'file event)))
+        (pos (cdr (assoc 'pos event)))
+        (id (cdr (assoc 'id event))))
+    ;; Check relative notifications (intervals before event times)
+    (dolist (time-info times)
+      (let ((timestamp-string (plist-get time-info :timestamp-string))
+            (event-time (plist-get time-info :time))
+            (timestamp-type (plist-get time-info :timestamp-type)))
+        (when (org-wild-notifier--has-timestamp timestamp-string)
+          (dolist (interval intervals)
+            (let ((notify-time (time-subtract event-time (seconds-to-time (* 60 interval)))))
+              (push (list :notify-at notify-time
+                          :event-time event-time
+                          :event-time-string timestamp-string
+                          :timestamp-type timestamp-type
+                          :minutes-before interval
+                          :type 'relative
+                          :title title
+                          :marker marker
+                          :file file
+                          :pos pos
+                          :id id
+                          :event event)
+                    notifications))))))
+    ;; Check absolute notifications (notify-at times)
+    (dolist (notify-time notify-at-times)
+      (push (list :notify-at notify-time
+                  :event-time nil
+                  :event-time-string nil
+                  :timestamp-type nil
+                  :minutes-before nil
+                  :type 'absolute
+                  :title title
+                  :marker marker
+                  :file file
+                  :pos pos
+                  :id id
+                  :event event)
+            notifications))
+    ;; Check day-wide notifications (overdue or day-wide events at configured times)
+    (when (org-wild-notifier-display-as-day-wide-event event)
+      (dolist (notify-time (org-wild-notifier--day-wide-alert-times-for-today))
+        (push (list :notify-at notify-time
+                    :event-time nil
+                    :event-time-string nil
+                    :timestamp-type nil
+                    :minutes-before nil
+                    :type 'day-wide
+                    :title title
+                    :marker marker
+                    :file file
+                    :pos pos
+                    :id id
+                    :event event)
+              notifications)))
+    (nreverse notifications)))
+
+(defun org-wild-notifier--deduplicate-markers (markers)
+  "Remove duplicate MARKERS pointing to the same buffer position.
+When a repeating task appears on multiple days in the agenda,
+the same org entry is returned multiple times with identical markers.
+This deduplicates based on (buffer . position) to ensure each org entry
+is only processed once."
+  (let ((seen (make-hash-table :test 'equal))
+        (result nil))
+    (dolist (marker markers)
+      (when (and marker (marker-buffer marker))
+        (let ((key (cons (marker-buffer marker) (marker-position marker))))
+          (unless (gethash key seen)
+            (puthash key t seen)
+            (push marker result)))))
+    (nreverse result)))
+
+(defun org-wild-notifier--gather-events-sync ()
+  "Synchronously gather events from agenda files.
+Returns a list of event alists suitable for notification processing."
+  (let ((org-agenda-use-time-grid nil)
+        (org-agenda-compact-blocks t)
+        (events nil))
+    (save-window-excursion
+      (org-agenda-list 2 (org-read-date nil nil "today"))
+      (with-current-buffer org-agenda-buffer-name
+        (setq events
+              (->> (org-split-string (buffer-string) "\n")
+                   (--map (plist-get
+                           (org-fix-agenda-info (text-properties-at 0 it))
+                           'org-marker))
+                   (-non-nil)
+                   (org-wild-notifier--apply-whitelist)
+                   (org-wild-notifier--apply-blacklist)
+                   (org-wild-notifier--deduplicate-markers)
+                   (-map 'org-wild-notifier--gather-info))))
+      (kill-buffer org-agenda-buffer-name))
+    events))
+
+;;;###autoload
+(defun org-wild-notifier-get-notifications-for-marker (marker)
+  "Get all notifications for the org entry at MARKER.
+Returns a list of notification plists, each containing:
+  :notify-at - Emacs time when notification should fire
+  :event-time - Emacs time of the event (nil for absolute/day-wide)
+  :event-time-string - Original timestamp string (nil for absolute/day-wide)
+  :timestamp-type - deadline, scheduled, or timestamp (nil for abs/day-wide)
+  :minutes-before - Minutes before event (nil for absolute/day-wide)
+  :type - Symbol: relative, absolute, or day-wide
+  :title - Event title string
+  :marker - Org marker for navigation/queries
+  :event - The full event alist"
+  (let ((event (org-wild-notifier--gather-info marker)))
+    (org-wild-notifier--get-notifications-for-event event)))
+
+;;;###autoload
+(defun org-wild-notifier-get-upcoming-notifications (&optional now)
+  "Get all notifications for upcoming events.
+If NOW is provided, use it as the current time for calculations.
+This allows querying notifications as if it were a different time.
+Returns a list of notification plists, each containing:
+  :notify-at - Emacs time when notification should fire
+  :event-time - Emacs time of the event (nil for absolute/day-wide)
+  :event-time-string - Original timestamp string (nil for absolute/day-wide)
+  :timestamp-type - deadline, scheduled, or timestamp (nil for abs/day-wide)
+  :minutes-before - Minutes before event (nil for absolute/day-wide)
+  :type - Symbol: relative, absolute, or day-wide
+  :title - Event title string
+  :marker - Org marker for navigation/queries
+  :event - The full event alist"
+  (org-wild-notifier--get-upcoming-notifications-1
+   (if now (lambda () now) (symbol-function 'current-time))))
+
+(defun org-wild-notifier--get-upcoming-notifications-1 (time-fn)
+  "Gather upcoming notifications using TIME-FN as `current-time'."
+  (cl-letf (((symbol-function 'current-time) time-fn))
+    (let* ((events (org-wild-notifier--gather-events-sync))
+           (all-notifications nil))
+      (dolist (event events)
+        (let ((event-notifications
+               (org-wild-notifier--get-notifications-for-event event)))
+          (setq all-notifications (append all-notifications event-notifications))))
+      all-notifications)))
+
 (provide 'org-wild-notifier)
 
 ;;; org-wild-notifier.el ends here
